const St = imports.gi.St;
const Main = imports.ui.main;
const GLib = imports.gi.GLib;
const GObject = imports.gi.GObject;
const Gio = imports.gi.Gio;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;
const Me = imports.misc.extensionUtils.getCurrentExtension();

let myPopup, panelButton, panelButtonText;

const MyPopup = GObject.registerClass(
	class MyPopup extends PanelMenu.Button {
		_init() {
			super._init(0);

			// Symbol in the top panel menu
			const icon = new St.Icon({
				icon_name: 'blueman-symbolic',
				style_class: 'system-status-icon'
			});
			
			this.add_child(icon)

			getDevices().forEach((device) => {
				let pmItem;

				// Check if a phone symbol should be added next to device. 
				//***
				const phoneSymbolProcess = Gio.Subprocess.new(
					// The program and command options are passed as a list of arguments
					['bluetoothctl', 'info', device.id],
					
					// The flags control what I/O pipes are opened and how they are directed
					Gio.SubprocessFlags.STDOUT_PIPE | Gio.SubprocessFlags.STDERR_PIPE
				);
				
				phoneSymbolProcess.communicate_utf8_async(null, null, (phoneSymbolProcess, res) => {
					try {
						let [, stdout, stderr] = phoneSymbolProcess.communicate_utf8_finish(res);
						
						// Failure
						if (!phoneSymbolProcess.get_successful())
							throw new Error(stderr);
						
						// Success
						pmItem = new PopupMenu.PopupImageMenuItem(
							device.name,
							stdout.indexOf("Icon: phone") >= 0 ? 'phone-symbolic' : ''
						);

						this.menu.addMenuItem(pmItem);
						addOnClickListener(pmItem);
					} catch (e) {
						logError(e);
					}
				});
				//***

				// Open the file dialog when a bluetooth device is clicked.
				//***
				function addOnClickListener(pmItem) {
					pmItem.connect('activate', () => {
						const proc = Gio.Subprocess.new(
							// The program and command options are passed as a list of arguments
							['bluetooth-sendto', `--device=${device.id}`, `--name=${device.name}`],
					
							// The flags control what I/O pipes are opened and how they are directed
							Gio.SubprocessFlags.STDOUT_PIPE | Gio.SubprocessFlags.STDERR_PIPE
						);
	
						proc.communicate_utf8_async(null, null, (proc, res) => {
							try {
								let [, , stderr] = proc.communicate_utf8_finish(res);
				
								// Failure
								if (!proc.get_successful())
									throw new Error(stderr);
							} catch (e) {
								logError(e);
							}
						});
					})
				}
				//***
			})
		}
	}
)

//Retrieve all paired Bluetooth devices
function getDevices() {
	const [, out, , ] = GLib.spawn_command_line_sync('bluetoothctl paired-devices');

	let deviceInfos = [];

	out.toString().split('\n').forEach(deviceInfo => {
		if (deviceInfo == '')
			return; //Because the last actual line ends with '\n', so there's a blank last line that we don't want to include.

		deviceInfos.push({
			//deviceInfo is formatted like 'Device 00:00:00:00:00:00 Name of Device'
			id: deviceInfo.substring(7, 25).trim(),
			name: deviceInfo.substring(25)});
		}
	);

	return deviceInfos;
}

function init() {
}

function enable() {
	myPopup = new MyPopup();
	Main.panel.addToStatusArea('myPopup', myPopup, 1)
}

function disable() {
	myPopup.destroy();
}